# PersistentDict is a dictionary that knows when it has been updated,
# so that changes are saved in the ZODB.
#
# It is similar to the Zope PersistentMapping class, but it does not
# hide keys that begin with an underscore, and keys are not restricted
# to being strings.
#
# This code is a copy of the Python UserDict.py, updated to support
# persistence.  My first thought was to subclass UserDict instead, but
# that would a pain because of the need to use inheritedAttribute with
# extension classes, and wouldn't be much of a win anyway.


from Persistence import Persistent

class PersistentDict (Persistent):

    # isinstance() doesn't work with extension classes, so we check
    # for this attribute instead
    isAPersistentDict = 1
    
    def __init__(self, dict=None):
        """Creates a PersistentDict object.  The optional 'dict'
        argument, which can be any dictionaryish object (a Python
        dictionary, another PersistentDict, a PersistentMapping, etc.)
        is used to populate the dictionary."""
        
        self.data = {}
        if dict is not None: self.update(dict)

    def __repr__(self): return repr(self.data)

    def __cmp__(self, dict):
        # untested, as __cmp__ is not getting called consistently in Zope 2.1.4 for some reason
        if type(dict) is type({}):
            return cmp(self.data, dict)
        elif hasattr(dict, 'isAPersistentDict'):
            return cmp(self.data, dict.data)
        else:
            return -1

    def __len__(self): return len(self.data)

    def __getitem__(self, key): return self.data[key]

    def __setitem__(self, key, item):
        self.data[key] = item
        self._p_changed = 1

    def __delitem__(self, key):
        del self.data[key]
        self._p_changed = 1

    def clear(self):
        self.data.clear()
        self._p_changed = 1

    def copy(self):
        return PersistentDict(self.data)

    def keys(self): return self.data.keys()

    def items(self): return self.data.items()

    def values(self): return self.data.values()

    def has_key(self, key): return self.data.has_key(key)

    def update(self, dict):
        if type(dict) is type({}):
            self.data.update(dict)
        elif hasattr(dict, 'isAPersistentDict'):
            self.data.update(dict.data)
        else:
            # we'll take any dictionaryish object
            d = self.data
            for k, v in dict.items():
                d[k] = v
        self._p_changed = 1

    def get(self, key, failobj=None):
        return self.data.get(key, failobj)
